
/* ************************************************************************* */
/*
 *	test_sha256.c
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */


#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include "system.h"
#include "util.h"
#include "util_tests.h"
#include "sha2.h"


#define dprintf(A) /* printf A */


/******************************************************************************/
/* file utilities */
/******************************************************************************/

int fcalc_hash(FILE *stream, char shash[2 * SHA256dlen + 1], int expected_len) {
	SHA256state st;
	uchar temp[SHA256dlen];
	int n;
	int ch;

	sha256init(&st);
	dprintf(("\nRead ->"));


	n = 0;
	ch = fgetc(stream);
	while(ch != EOF && expected_len > 0) {
		unsigned int uch1;
		unsigned int uch2;

		uch1 = unhex_char(ch);
		if (uch1 > 16) {
			break;
		}

		ch = fgetc(stream);
		uch2 = unhex_char(ch);
		if (uch2 > 16) {
			break;
		}


		temp[n] = (uch1 << 4) | uch2;
		dprintf(("%02hx", temp[n]));

		n++;
		if (n >= SHA256dlen) {
			sha256(&st, temp, (unsigned int)n);
			n = 0;
		}
		expected_len--;

		ch = fgetc(stream);
	}
	dprintf(("<-\n"));
	sha256(&st, temp, (unsigned int)n);

	sha256finish(&st, temp);
	hex(temp, SHA256dlen, shash);

	/* read to eol */
	fget_line(stream, NULL, 0);
	return 1;
}

int fget_hash(FILE *stream, char shash[2 * SHA256dlen + 1]) {
	int n;
	int ch;

	n = 0;
	ch = fgetc(stream);
	while(ch != EOF && n < 2 * SHA256dlen) {
		if ((ch < '0' || ch > '9') && (ch < 'a' || ch > 'f') && (ch < 'A' && ch > 'F')) {
			break;
		}
		shash[n] = (char)ch;

		n++;
		ch = fgetc(stream);
	}
	shash[n] = 0;

	/* read to eol */
	fget_line(stream, NULL, 0);

	if (n != 2 * SHA256dlen) {
		return 0;
	}
	else {
		return 1;
	}
}

int fget_tag(FILE *stream, char *dest, int dest_len) {
	int ch;
	int n;

	dest[0] = 0;
	dest_len--;


	/* scan and read to first space */
	n = 0;
	ch = fgetc(stream);
	while (ch != EOF && ch != ' ' && ch != '\n') {
		dest[n] = ch;
		ch = fgetc(stream);
		n++;
		if (n >= dest_len) {
			dest[0] = 0;
			return 1;
		}
	}
	if (ch == EOF) {
		dest[0] = 0;
		return 0;
	}
	if (ch != ' ') {
		ungetc(ch, stream);
		dest[0] = 0;
		return 1;
	}

	/* store next three chars */
	dest[n] = ch;
	n++;
	if (n >= dest_len) {
		dest[0] = 0;
		return 1;
	}

	/* two */
	ch = fgetc(stream);
	if (ch == EOF) {
		dest[0] = 0;
		return 0;
	}
	dest[n] = ch;
	n++;
	if (n >= dest_len) {
		dest[0] = 0;
		return 1;
	}

	/* three */
	ch = fgetc(stream);
	if (ch == EOF) {
		dest[0] = 0;
		return 0;
	}
	dest[n] = ch;
	n++;
	if (n >= dest_len) {
		dest[0] = 0;
		return 1;
	}

	/* return valid */
	dest[n] = 0;
	return 1;
}


/******************************************************************************/
/* main and usage */
/******************************************************************************/

void usage(void)
{
	fprintf(stderr, "Usage: sha256-test [filename]\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "This program reads a standard test vector file, calculates\n");
	fprintf(stderr, "checksums, and displays success or failure for each vector.\n");
}

int main(int argc, char *argv[])
{
	FILE *stream;
	char tag_space[100];
	char ref_hash[2*SHA256dlen+1];
	char tst_hash[2*SHA256dlen+1];
	int len;
	int fail = 0;

	if (argc <= 1 || (argv[1] != NULL && (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0))) {
		usage();
		return 1;
	}
	
	stream = fopen(argv[1], "r");
	if (stream == NULL) {
		printf("Unable to open file '%s'.\n", argv[1]);
		return 1;
	}

	len = 1;
	while(fget_tag(stream, tag_space, sizeof(tag_space))) {
		dprintf(("\nGot tag '%s'\n", tag_space));
		if (strcmp(tag_space, "Len = ") == 0) {
			fget_line(stream, tag_space, sizeof(tag_space));
			len = atoi(tag_space);
		}
		else if (strcmp(tag_space, "Msg = ") == 0) {
			if (!fcalc_hash(stream, tst_hash, len)) {
				printf("Error hashing file '%s'.\n", argv[1]);
			}
			else {
				printf("Result: %s ", tst_hash);
			}
		}
		else if (strcmp(tag_space, "MD = ") == 0) {
			if (!fget_hash(stream, ref_hash)) {
				printf("Error reading reference hash value.\n");
			}
			else {
				printf("Expected: %s", ref_hash);
				if (strcmp(ref_hash, tst_hash) != 0) {
					printf("  FAIL");
					fail = 1;
				}
				printf("\n");
			}
		}
		else {
			/* read to eol */
			fget_line(stream, tag_space, sizeof(tag_space));
		}
	}
	fclose(stream);

	return fail;
}

