
/* ************************************************************************* */
/*
 *	test_vector_data.c
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */

#include <stdlib.h>
#include <string.h>

#include "system.h"
#include "uuid_hash.h"
#include "util_tests.h"

#define dprintf(A) /* printf A */	/* debug printing macro */


/******************************************************************************/
/* uuid display/generate helpers */
/******************************************************************************/

void print_uuid_parms(struct test_data *td) {
	printf("patient record number %d\n", td->record_number);
	printf("patient_name          %s\n", td->patient_name);
	if (td->patient_ssn) {
		printf("patient_ssn           %09d\n", td->patient_ssn);
	}
	printf("patient_gender        %d\n", td->patient_gender);
	printf("patient_birth_year    %d\n", td->patient_year);
	printf("patient_birth_month   %d\n", td->patient_month);
	printf("patient_birth_day     %d\n", td->patient_day);

	if (td->policy_ssn) {
		printf("policy_name           %s\n", td->policy_name);
		printf("policy_ssn            %09d\n", td->policy_ssn);
		printf("policy_gender         %d\n", td->policy_gender);
		printf("policy_birth_year     %d\n", td->policy_year);
		printf("policy_birth_month    %d\n", td->policy_month);
		printf("policy_birth_day      %d\n", td->policy_day);
	}
	printf("patient_uuid          %s\n", td->uuid);
}


/******************************************************************************/
/* demo routine - patient SSN available */
/******************************************************************************/

int read_patient_record(FILE *stream, struct test_data *td) {
	init_test_data(td);
	char temp[1000];

/* sample data record from the input file:
data_record           14
patient_name          Dominique
patient_gender        2
patient_birth_year    1934
patient_birth_month   11
patient_birth_day     26
policy_name           Crystal
policy_ssn            111222014
policy_gender         2
policy_birth_year     1910
policy_birth_month    1
policy_birth_day      1
patient_uuid          5p2dtl3u2nke
*/

	while(fget_line(stream, temp, sizeof(temp))) {
		dprintf(("Got line '%s'\n", temp));

		char *next = next_arg(temp);
		if (*temp == 0 || *temp == '#') {
			/* do nothing - these are comment lines */
		}
		else if (next == NULL) {
			printf("Error in input file - got bad line '%s'.\n", temp);
			return 0;
		}

		else if (wordcmp(temp, "data_record")) {
			td->record_number = atoi(next);
		}

		else if (wordcmp(temp, "patient_name")) {
			strxcpy(td->patient_name, next, sizeof(td->patient_name));
		}
		else if (wordcmp(temp, "patient_ssn")) {
			td->patient_ssn = atoi(next);
		}
		else if (wordcmp(temp, "patient_gender")) {
			td->patient_gender = atoi(next);
		}
		else if (wordcmp(temp, "patient_birth_year")) {
			td->patient_year = atoi(next);
		}
		else if (wordcmp(temp, "patient_birth_month")) {
			td->patient_month = atoi(next);
		}
		else if (wordcmp(temp, "patient_birth_day")) {
			td->patient_day = atoi(next);
		}

		else if (wordcmp(temp, "policy_name")) {
			strxcpy(td->policy_name, next, sizeof(td->patient_name));
		}
		else if (wordcmp(temp, "policy_ssn")) {
			td->policy_ssn = atoi(next);
		}
		else if (wordcmp(temp, "policy_gender")) {
			td->policy_gender = atoi(next);
		}
		else if (wordcmp(temp, "policy_birth_year")) {
			td->policy_year = atoi(next);
		}
		else if (wordcmp(temp, "policy_birth_month")) {
			td->policy_month = atoi(next);
		}
		else if (wordcmp(temp, "policy_birth_day")) {
			td->policy_day = atoi(next);
		}

		else if (wordcmp(temp, "patient_uuid")) {
			strxcpy(td->uuid, next, sizeof(td->uuid));
			return 1;
		}
	}

	return 0;
}

static int load_and_check_data_set(const char *filename, int quiet) {
	struct test_data td;
	FILE *stream;
	int counter;
	int fail_counter;

	stream = fopen(filename, "r");
	if (stream == NULL) {
		printf("Unable to open input file '%s'.  Aborting.\n", filename);
		exit(1);
	}

	if (!quiet) {
		printf("Reading patient records from input file '%s'.\n", filename);
	}

	fail_counter = 0;
	counter = 0;
	init_test_data(&td);
	while(read_patient_record(stream, &td)) {
		char temp[UUID_HASH_STRING_LENGTH + 1];

		counter++;
		generate_test_uuid(&td, temp);
		dprintf(("Generated UUID '%s'\n", temp));

		if (strcmp(temp, td.uuid) != 0) {
			if (!quiet) {
				printf("\n");
				printf("FAIL - read record data:\n");
				print_uuid_parms(&td);
				printf("Calculated UUID '%s' does not match.\n", temp);
			}
			fail_counter++;
		}
	}

	if (fail_counter > 0) {
		printf("FAIL - %d out of %d records failed to match UUIDs.\n", fail_counter, counter);
	}
	else {
		if (!quiet) {
			printf("%d records tested, all UUIDs matched.\n", counter);
		}
	}
	return fail_counter;
}


/******************************************************************************/
/* main routine and usage */
/******************************************************************************/

static void dump_test_usage(const char *name) {
	printf("Usage:\n");
	printf("%s <-q> [test vector filename]\n", name);
	printf("\n");
	printf("This program loads test vectors from a given file name and checks to\n");
	printf("see that the generated UUID matches that in the test vector file.\n");
	printf("\n");
	printf("Test vectors can be produced by the 'gen_test_data' program.\n");
	printf("\n");
	printf("What file would you like to check?\n");
}

int main(int argc, char *argv[]) {
	const char *filename = NULL;
	int quiet = 0;

	if (argc <= 1 || (argc == 2 && strcmp(argv[1], "-h") == 0)) {
		dump_test_usage(argv[0]);
		return 0;
	}
	if (strcmp(argv[1], "-q") == 0) {
		if (argc == 2) {
			dump_test_usage(argv[0]);
			return 0;
		}
		quiet = 1;
		filename = argv[2];
	}
	else {
		filename = argv[1];
	}

	return load_and_check_data_set(filename, quiet);
}

